Package: webfakes
Title: Fake Web Apps for HTTP Testing
Version: 1.5.0
Authors@R: c(
    person("Gábor", "Csárdi", , "csardi.gabor@gmail.com", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49")),
    person(, "Civetweb contributors", role = "ctb",
           comment = "see inst/credits/ciwetweb.md"),
    person(, "Redoc contributors", role = "ctb",
           comment = "see inst/credits/redoc.md"),
    person("L. Peter", "Deutsch", role = "ctb",
           comment = "src/md5.h"),
    person("Martin", "Purschke", role = "ctb",
           comment = "src/md5.h"),
    person(, "Aladdin Enterprises", role = "cph",
           comment = "src/md5.h"),
    person("Maëlle", "Salmon", , "maelle.salmon@yahoo.se", role = "ctb",
           comment = c(ORCID = "0000-0002-2815-0399"))
  )
Description: Create a web app that makes it easier to test web clients
    without using the internet. It includes a web app framework with path
    matching, parameters and templates. Can parse various 'HTTP' request
    bodies. Can send 'JSON' data or files from the disk. Includes a web
    app that implements the 'httpbin.org' web service.
License: MIT + file LICENSE
URL: https://webfakes.r-lib.org/, https://github.com/r-lib/webfakes
BugReports: https://github.com/r-lib/webfakes/issues
Depends: R (>= 3.6)
Imports: stats, tools, utils
Suggests: brotli, callr, covr, curl, digest, glue, httpuv, httr,
        jsonlite, processx, testthat (>= 3.0.0), withr, xml2, zip (>=
        2.3.0)
Biarch: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-28
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-16 16:57:00 UTC; gaborcsardi
Author: Gábor Csárdi [aut, cre],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>),
  Civetweb contributors [ctb] (see inst/credits/ciwetweb.md),
  Redoc contributors [ctb] (see inst/credits/redoc.md),
  L. Peter Deutsch [ctb] (src/md5.h),
  Martin Purschke [ctb] (src/md5.h),
  Aladdin Enterprises [cph] (src/md5.h),
  Maëlle Salmon [ctb] (ORCID: <https://orcid.org/0000-0002-2815-0399>)
Maintainer: Gábor Csárdi <csardi.gabor@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-16 21:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-17 02:15:11 UTC; unix
Archs: webfakes.so.dSYM
