# Generated by roxygen2: do not edit by hand

S3method(format,webfakes_app)
S3method(format,webfakes_app_process)
S3method(format,webfakes_regexp)
S3method(format,webfakes_request)
S3method(format,webfakes_response)
S3method(print,webfakes_app)
S3method(print,webfakes_app_process)
S3method(print,webfakes_regexp)
S3method(print,webfakes_request)
S3method(print,webfakes_response)
export(git_app)
export(http_time_stamp)
export(httpbin_app)
export(local_app_process)
export(mw_cgi)
export(mw_cookie_parser)
export(mw_etag)
export(mw_json)
export(mw_log)
export(mw_multipart)
export(mw_range_parser)
export(mw_raw)
export(mw_static)
export(mw_text)
export(mw_urlencoded)
export(new_app)
export(new_app_process)
export(new_regexp)
export(oauth2_httr_login)
export(oauth2_login)
export(oauth2_resource_app)
export(oauth2_third_party_app)
export(server_opts)
export(tmpl_glue)
useDynLib(webfakes, .registration = TRUE, .fixes = "c_")
