#' @title Happy HTTP testing with webfakes
#' @name introduction
#' @section Happy HTTP testing with webfakes:
#'
#' ```{r child = "vignettes/introduction.Rmd"}
#' ```
NULL

#' @title webfakes glossary
#' @name glossary
#' @section Webfakes glossary:
#'
#' ```{r child = "vignettes/glossary.Rmd"}
#' ```
NULL

#' @title How to use webfakes in your tests
#' @name how-to
#' @section How to use webfakes in your tests:
#'
#' ```{r child = "vignettes/how-to.Rmd"}
#' ```
NULL
